/*
 * po_worker.cpp
 *
 *  Last modified: January 18, 2011
 *         Author: Marcus Lechner
 */

// reading a text file
#define _MULTI_THREADED
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <map>
#include <algorithm>
#include <cmath>
#include <vector>
#include <stack>
#include <pthread.h>
#include <iomanip>
using namespace std;

// Functions
void print_list(vector<vector<unsigned int> > , unsigned int, double);
void bulid_id_to_protein(void);
void color_it();
unsigned int read_fasta(string);
unsigned int read_blast(string, string);
map<unsigned int, map<unsigned int, pair<double, double> > > read_blast_data(
		const string);
void get_bidirectionals(map<unsigned int, map<unsigned int,
		pair<double, double> > > , map<unsigned int, map<unsigned int, pair<
		double, double> > > );
void tokenize(const string&, vector<string>&, const string&);
double string2double(string);
string full_path2filename(string);
void setup_species(string);
void stats(float, float);
void *thread_blast_reader (void *);

// Algebraic functions
double getConnectivity(vector<unsigned int> nodes);
vector<double> getY(double max, vector<double> , vector<double> , vector<unsigned int> );
unsigned int max_deg(vector<unsigned int> );
vector<double> get_new_x(vector<double> , vector<unsigned int>);
vector<double> generate_random_vector(unsigned int );
vector<double> makeOrthogonal(vector<double> );
vector<double> nomalize(vector<double> , double*);
void splitGroups(vector<double>, vector<unsigned int>);
void remove_excluded_links(map<unsigned int, bool>);
void store_parameters();


// Global variables
map<string, pair<unsigned int, unsigned int> > species; // species_file	-> species_id, #sequences
map<string, pair<unsigned int, unsigned int> > proteins; // protein_name	-> protein_id, size
vector<unsigned int> protein_species_mapping; // protein_id	-> belongs to species_id
vector<string> id_to_protein;
vector<vector<unsigned int> > graph;
unsigned int species_counter = 0;
unsigned int protein_counter = 0;
fstream log_out;
vector<unsigned int> mapping;

// Parameters
vector<string> filenames;
double coverage = 0.5;
int selfblast = 0;
int percent_identity = 25;
bool unambigous = false;
string blastdir = ".";
bool logfile = false;
bool verbose = false;
bool inc_pairs = false;
bool debug = false;
double sim = 0.95;
double evalue = 1;
float last_stat = -1;
float con_threshold = 0.1;
string blastversion = "";

unsigned int nTHREADS = 1, nTHREADSio = 1;
pthread_mutex_t update_mutex = PTHREAD_MUTEX_INITIALIZER;

int main(int argc, char *argv[]) {
	string file_list = "";
	bool iolimit = true;
	try {
		// Read parameters
		int paras;
		for (paras = 1; paras < argc; paras++) {
			string parameter = string(argv[paras]);
			if (parameter.substr(0, 1) != "-" && file_list.empty()) {
				break;
			}
			if (parameter == "-blastdir") {
				paras++;
				blastdir = string(argv[paras]);
			} else if (parameter == "-log") {
				paras++;
				logfile = true;
				log_out.open(argv[paras], ios::out);
			} else if (parameter == "-selfblast") {
				selfblast = 1;
			} else if (parameter == "-verbose") {
				verbose = true;
			} else if (parameter == "-debug") {
				debug = true;
			}
		    else if (parameter == "-threads") {
		    	paras++;
		    	nTHREADS = string2double(string(argv[paras]));
		    	if (nTHREADS < 1) {nTHREADS = 1;}
		    }
			else if (parameter == "-unambigous") {
				unambigous = true;
			}
			else if (parameter == "-noiolimit") {
				iolimit = false;
			} else if (parameter == "-percent_identity") {
				paras++;
				percent_identity = string2double(string(argv[paras]));
			} else if (parameter == "-coverage") {
				paras++;
				coverage = string2double(string(argv[paras]));
			}  else if (parameter == "-connectivity") {
				paras++;
				con_threshold = string2double(string(argv[paras]));
			}  else if (parameter == "-m") {
				paras++;
				sim = string2double(string(argv[paras]));
			} else if (parameter == "-list") {
				paras++;
				file_list = string(argv[paras]);
			} else if (parameter == "-blastversion") {
				paras++;
				blastversion = string(argv[paras]);
			} else if (parameter == "-evalue") {
							paras++;
							evalue  = string2double(string(argv[paras]));
			}
			else if (parameter == "-include_pairs") {
				inc_pairs = true;
			}
			else {
				throw string("Unknown option '" + string(argv[paras]) + "'!");
			}
		}

		nTHREADSio = nTHREADS;
    	if (nTHREADSio > 4 && iolimit) {nTHREADSio = 4; if (verbose | debug) {cerr << "Limiting #I/O threads to " << nTHREADSio << endl;}}

		if (verbose && debug) {
			cerr << "Warning, -debug and -verbose cannot be used together! Using debug only." << endl;
			verbose = false;
		}

		// Get list of protein databases
		if (!file_list.empty()) {
			// fill filenames from list-file
			setup_species(file_list);
		} else {
			// fill filenames from program arguments
			for (int i = paras; i < argc; i++) {
				filenames.push_back(argv[i]);
			}
		}
		if (filenames.size() <= 1) {
			throw string("At least two files are needed!");
		}
		filenames.resize(filenames.size());	// sIze

		if (logfile) {
			log_out << "#species\tprotein id\tinternal id" << endl;
		}

		cerr << "Building indexes..." << endl;
		string number_header = "";
		cout << "#species\tproteins\talg.-conn.\t";

		// Read map, check fasta files and write output-header
		for (unsigned int i = 0; i < filenames.size(); i++) {
			species[filenames[i]] = pair<int, int> (species_counter++,
			read_fasta(filenames[i]));				// protein_species_mapping
			cout << full_path2filename(filenames[i]);
			if (i != filenames.size() - 1) {
				cout << "\t";
			}
			if (verbose) {
				stats(i+1,filenames.size());
			}
		}
		protein_species_mapping.resize(protein_species_mapping.size()); // sIze
		if (verbose) {cerr << endl;last_stat = -1;}

		cout << "\n" << "#" << species_counter << "\t" << protein_counter
				<< "\t-";

		for (unsigned int i = 0; i < filenames.size(); i++) {
			cout << "\t";
			cout << species[filenames[i]].second;
		}
		cout << "\n";

		if (logfile) {
			log_out
					<< "#protein_a\tprotein_b\tE-value a2b\tbit score a2b\tE-value b2a\tbit score b2a\t"
					<< endl;
		}
		cerr << "Reading blast output..." << endl;
		// Aquire graph from blasts
		graph = vector<vector<unsigned int> > (protein_counter, vector<
				unsigned int> (0, 0));

		// Thread vector
		pthread_t thread[nTHREADSio];
		unsigned int thread_data[nTHREADSio][2];
		unsigned int threads = 0;

		unsigned int all_runs = (filenames.size()*(filenames.size()-1))/2;
		if (selfblast) all_runs = (filenames.size()*(filenames.size())+filenames.size())/2;

		unsigned int done_counter = 0;
		for (unsigned i = 0; i < filenames.size() - 1 + selfblast; i++) {
			for (unsigned j = i + 1 - selfblast; j < filenames.size(); j++) {
				// Threaded reading
				if (threads >= nTHREADSio) {
					// Join thread block
					 for(unsigned int k=0; k < threads; k++) {
						 pthread_join( thread[k], NULL);
						 if (verbose) {done_counter++;}
					 }
					 if (verbose) {stats(done_counter,all_runs);}
					 threads = 0;
				}
				// run a new thread
				thread_data[threads][0] = i;
				thread_data[threads][1] = j;

				 /* Create independent threads each of which will execute function */
				 pthread_create( &thread[threads], NULL, thread_blast_reader, (void*) thread_data[threads]);
				 threads++;
			}
		}
		// Join the rest
		for(unsigned int k=0; k < threads; k++) {
			 pthread_join( thread[k], NULL);
			 if (verbose) {done_counter++;}
		}
		if (verbose) {stats(done_counter,all_runs);}


		graph.resize(graph.size()); // sIze
		if (verbose) {
			cerr << endl;
			last_stat = -1;
		}

		cerr << "Building maps..." << endl;
		// Init back mapping
		bulid_id_to_protein();

		cerr << "Determining (co-)orthologous groups..." << endl;
		// Coloring and output
		color_it();

		store_parameters();

	} catch (string e) {
		cerr << endl << "Error: " << e << endl;
		if (logfile) {
			log_out.close();
			logfile = false;
		}
		exit(EXIT_FAILURE);
	}

	if (logfile) {
		log_out.close();
		logfile = false;
	}
	cerr << "Done." << endl;
	return EXIT_SUCCESS;
}

void store_parameters() {
	   cout << "#version: Proteinortho 4.20"
			<< ", blast=" << blastversion
			<< ", E-value=" << evalue
			<< ", alg.-conn.=" << con_threshold
			<< ", coverage=" << coverage
			<< ", percent_identity=" << percent_identity
			<< ", adaptive_similarity=" << sim
			<< ", inc_pairs=" << inc_pairs
			<< ", selfblast=" << selfblast
			<< ", unambiguous=" << unambigous
			<< endl;
}

void *thread_blast_reader( void *ptr )
{
	   int *job;
	   job = (int *) ptr;
	   read_blast(filenames[job[0]], filenames[job[1]]);
	   return NULL;
}

string full_path2filename(string full) {
	int found = full.find_last_of('/', full.size());
	if (found < 1) {
		return full;
	}
	found++;
	return full.substr(found, full.size() - found);
}

void bulid_id_to_protein() {
	id_to_protein = vector<string> (protein_counter);

	while (!proteins.empty()) {
		id_to_protein[(*proteins.begin()).second.first]
				= (*proteins.begin()).first;
		proteins.erase(proteins.begin());
	}

	//	for (map<string, pair<unsigned int, unsigned int> >::iterator ii = proteins.begin(); ii != proteins.end(); ii++) {
	//		id_to_protein[(*ii).second.first] = (*ii).first;
	//	}
	proteins.clear();
	id_to_protein.resize(id_to_protein.size()); // sIze
}

void color_it() {
	mapping = vector<unsigned int> (protein_counter);
	stack<unsigned int> todo;
	vector<bool> done = vector<bool> (protein_counter, false);

	// foreach protein
	for (unsigned int protein = 0; protein < protein_counter;) {
//		if (debug) cerr << "Running: " << protein;
		// skip if already colored
		if (done[protein]) {
			protein++;
//			if (debug) cerr << " done" << endl;
			continue;
		}
//		if (debug) cerr << " work" << endl;
		// add to list
		todo.push(protein);

		// mark protein as done
		done[protein] = true;

		// Start new map and list results
		vector<unsigned int> included_nodes;
		vector<vector<unsigned int> > list = vector<vector<unsigned int> > (
				species_counter, vector<unsigned int> (0, 0));
		unsigned int allover_hits = 0; // For confidence value maybe bigger -> double rounding probs

		// while stack is not empty
		while (!todo.empty()) {
			// get next protein
			unsigned int next = todo.top();
			todo.pop();
			//				cout << "Locate: " << next << " -> " << protein_species_mapping[next] << endl;
			// note this protein
			list[protein_species_mapping[next]].push_back(next);
			included_nodes.push_back(next);
			//	cerr << "F " << id_to_protein[next] << endl;
			//	if (id_to_protein[next] == "gi|47529966|ref|YP_021315.1|") {cerr << "FOUND FATHER" << endl;}

			// for all targets
			//	cerr << "Size: " << graph[next].size() << endl;
			allover_hits += graph[next].size();
			for (unsigned int to = 0; to < graph[next].size(); to++) {
				//		cerr << "F " << id_to_protein[graph[next][to]] << endl;

				//					if (id_to_protein[graph[next][to]] == "gi|47529966|ref|YP_021315.1|") {cerr << "FOUND CHILD" << endl;}
				// skip if colored before
				if (done[graph[next][to]])
					continue;
				todo.push(graph[next][to]);
				// mark protein as done
				done[graph[next][to]] = true;
			}
		}

		// no groups with min. edges, including 1 and 2 sized sets
		// Trees (min)
		double min = 2 * (included_nodes.size() - 1);
		double max = included_nodes.size() * (included_nodes.size() - 1);
		if (allover_hits > min or (inc_pairs and min==2)) {
			// getConnectivity cleans the graph if successful
			double connectivity = 1;

			if (allover_hits < max) {
				connectivity = getConnectivity(included_nodes);
			}
			if (debug) {cerr << "C: " << connectivity << "\t#" << included_nodes.size() << endl;}
			// Negativ -> The group should be splitted

			if (connectivity < 0) {
				// remove done marker
				for (unsigned int k = 0; k < included_nodes.size(); k++) {
					done[included_nodes[k]] = false;
				}
				// retry with splitted group
				continue;
			}
		    else {
		    	print_list(list, included_nodes.size(), connectivity);
		    	if (verbose) {stats(protein,protein_counter);}
		    }
	    }
		protein++;

	  // Clean subgraph
	  for (unsigned int i = 0; i < included_nodes.size(); i++) {
		  graph[included_nodes[i]].clear();
	  }

		}

		mapping.clear();
	    if (verbose) {stats(protein_counter,protein_counter);cerr << endl;last_stat = -1;}
}

void print_list(vector<vector<unsigned int> > list, unsigned int n, double connectivity) {
	// Check for paralogs
	if (unambigous) {
		for (unsigned int i = 0; i < species_counter; i++) {
			if (list[i].size() > 1) {
				return;
			}
		}
	}

	// Count species
	unsigned int inc_specs = 0;
	for (unsigned int i = 0; i < species_counter; i++) {
		if (list[i].size() > 0) {
			inc_specs++;
		}
	}
	if ((inc_specs < 2 and selfblast == 0) or inc_specs == 0) {
		return;
	} // only paralogs detected

	// Line start
	cout << inc_specs << "\t" << n << "\t" << setprecision (3) << connectivity << "\t"; // << allover_hits << "\t";

	// Print line
	for (unsigned int i = 0; i < species_counter; i++) {
		if (list[i].size() == 0)
			cout << "*";
		for (unsigned int j = 0; j < list[i].size(); j++) {
			cout << id_to_protein[list[i][j]];
			if (j != list[i].size() - 1)
				cout << ",";
		}
		if (i != species_counter - 1)
			cout << "\t";
	}
	cout << endl;
}

unsigned int read_blast(string file_name_a, string file_name_b) {

	if (debug) {
		cerr << "Reading blast output for '" << file_name_a << "' and '" << file_name_b << "'" << endl;
	}

	try {
	if (debug) {
		cerr << "Source 1: " << blastdir + "/" + full_path2filename(file_name_a) + "." + full_path2filename(file_name_b) + ".bla" << endl;
	}
	// hitmap -> id-a -> id-b -> evalue, bitscore
	map<unsigned int, map<unsigned int, pair<double, double> > > hits_a =
			read_blast_data(blastdir + "/" + full_path2filename(file_name_a)
					+ "." + full_path2filename(file_name_b) + ".bla");

	map<unsigned int, map<unsigned int, pair<double, double> > > hits_b;
	if (file_name_a.compare(file_name_b) == 0) {
		hits_b = hits_a;
	}
	else {

	if (debug) {
		cerr << "Source 2: " << blastdir + "/" + full_path2filename(file_name_b) + "." + full_path2filename(file_name_a) + ".bla" << endl;
	}
	hits_b =
			read_blast_data(blastdir + "/" + full_path2filename(file_name_b)
					+ "." + full_path2filename(file_name_a) + ".bla");
	}

	pthread_mutex_lock(&update_mutex);
	get_bidirectionals(hits_a, hits_b);
	pthread_mutex_unlock(&update_mutex);
	hits_a.clear();
	hits_b.clear();
	}
	catch (string e) {
		cerr << "Error: " << e << endl;
		exit(EXIT_FAILURE);
	}

	return 1;
}

void get_bidirectionals(map<unsigned int, map<unsigned int,
		pair<double, double> > > hits_a, map<unsigned int, map<unsigned int,
		pair<double, double> > > hits_b) {


	// cleanup
	for (map<unsigned int, map<unsigned int, pair<double, double> > >::iterator
			ii = hits_a.begin(); ii != hits_a.end(); ii++) {
		//			cerr << "Key:\t" << (*ii).first << "\tBest: " << best[(*ii).first].first << endl;
		for (map<unsigned int, pair<double, double> >::iterator sii =
				(*ii).second.begin(); sii != (*ii).second.end(); sii++) {
			//				cerr << "Found: " << (*ii).first << " to " << (*sii).first << endl;
			if ((*sii).second.first == -1) {
				continue;
			} // Marked for deletion
			if (hits_b[(*sii).first].count((*ii).first) > 0) {
				if (hits_b[(*sii).first][(*ii).first].first == -1) {
					continue;
				} // Marked for deletion
				graph[(*ii).first].push_back((*sii).first); // add list
				graph[(*sii).first].push_back((*ii).first); // and revers
				// LOGFILE
				//	write log-file
				//	logSpec a	Spec b	prot a	prot b	evalue	bitscore
				if (logfile) {
					log_out << (*ii).first << "\t" << (*sii).first << "\t"
							<< (*sii).second.first << "\t"
							<< (*sii).second.second << "\t"
							<< hits_b[(*sii).first][(*ii).first].first << "\t"
							<< hits_b[(*sii).first][(*ii).first].second << endl;
				}
				// cerr << "Detected: " << (*ii).first << "\t" << (*sii).first << endl;
			}
		}
	}

}

map<unsigned int, map<unsigned int, pair<double, double> > > read_blast_data(
		const string file) {

	map<unsigned int, map<unsigned int, pair<double, double> > > hits; // hitmap -> id-a -> id-b -> evalue, bitscore
	map<unsigned int, pair<double, double> > best; // evalue, bitscore
	string line;
	ifstream blast_a_file(file.c_str());
	if (blast_a_file.is_open()) {
		while (!blast_a_file.eof()) {
			getline(blast_a_file, line);
			vector<string> fields;
			tokenize(line, fields, "\t");
			// empty or illegal line
			if (fields.size() < 12) {
				//		cerr << "Skipp: Fields" << endl;
				continue;
			}
			// less than percent_identity% seq id
			if (string2double(fields[2]) < percent_identity) {
				//		cerr << "Skipp: Coverage" << endl;
				continue;
			}
			// less than 50% coverage
			if (string2double(fields[3]) < proteins[fields[1]].second*coverage + 0.5) {
				//	cerr << "Skipp: Coverage" << endl;
				continue;
			}

			// Protein is itself
			if (selfblast == 1 && fields[0].compare(fields[1]) == 0) {
				continue;
			}

			if (proteins.count(fields[0]) == 0) {
				throw "Protein '" + fields[0] + "' refered in '" + file
						+ "' is not in the database!";
			}
			if (proteins.count(fields[1]) == 0) {
				throw "Protein '" + fields[1] + "' refered in '" + file
						+ "' is not in the database!";
			}

			//		cerr << "||'" << fields[0] << "'\t'" << fields[1] << "'" << endl;

			unsigned int p_a = proteins[fields[0]].first;
			unsigned int p_b = proteins[fields[1]].first;
			double evalue = string2double(fields[10]);
			double bitscore = string2double(fields[11]);

			//			cout << "||" << proteins[fields[0]].first << "\t" << proteins[fields[1]].first << endl;

			//			cout << "Found:\t" << p_a << "->" << p_b << "\tevalue:\t" << evalue << endl;

			if (hits.count(p_a) == 0) { // no entry defined
				hits[p_a] = map<unsigned int, pair<double, double> > ();
				best[p_a] = pair<double, double> (evalue, bitscore); // e-value
				//		cerr << "Best: " << evalue << "\t" << bitscore << endl;
			}

			// addaptive
			if (bitscore / best[p_a].second < sim) {
				// cerr << "Skipp: Sim" << endl;
				continue;
			}
			// better exists
			if (hits[p_a].count(p_b) > 0 and hits[p_a][p_b].first <= evalue) {
				//	cerr << "Skipp: Evalue" << endl;
				continue;
			}

			hits[p_a][p_b] = pair<double, double> (evalue, bitscore);
			//		cerr << "Add: " << hits[p_a][p_b].first << endl;
			if (best[p_a].first > evalue) {
				//				cerr << "Best:" << fields[0] << "\t" << best[p_a].first << "\tnew\t" << evalue << endl;
				best[p_a] = pair<double, double> (evalue, bitscore);
			}
		}
		blast_a_file.close();
	} else {
		throw "Unable to open file " + file;
	}

	// cleanup
	for (map<unsigned int, map<unsigned int, pair<double, double> > >::iterator
			ii = hits.begin(); ii != hits.end(); ii++) {
		//		cerr << "Key:\t" << (*ii).first << "\tBest: " << best[(*ii).first].first << endl;
		for (map<unsigned int, pair<double, double> >::iterator sii =
				(*ii).second.begin(); sii != (*ii).second.end(); sii++) {
			//			cerr << "Child:\t" << (*sii).first << " -> " << (*sii).second.first << endl;
			//			cerr << (*sii).second.second << "/" << best[(*ii).first].second << "=" << double((*sii).second.second)/double(best[(*ii).first].second) << endl;
			if ((*sii).second.second / best[(*ii).first].second < sim) {
				//			cerr << "Key:\t" << (*ii).first << "\tBest: " << best[(*ii).first].first << endl;
				//			cerr << "Child:\t" << (*sii).first << " -> " << (*sii).second.first << endl;
				//			cerr << (*sii).second.second << "/" << best[(*ii).first].second << "=" << double((*sii).second.second)/double(best[(*ii).first].second) << endl;
				//			cerr << "Removed because better match available" << endl;
				(*sii).second.first = -1;
			}
		}
	}


	return hits;
}

double string2double(string str) {
	istringstream buffer(str);
	double value;
	buffer >> value;
	return value;
}

void tokenize(const string& str, vector<string>& tokens,
		const string& delimiters = "\t") {
	// Skip delimiters at beginning.
	string::size_type lastPos = str.find_first_not_of(delimiters, 0);
	// Find first "non-delimiter".
	string::size_type pos = str.find_first_of(delimiters, lastPos);

	while (string::npos != pos || string::npos != lastPos) {
		// Found a token, add it to the vector.
		tokens.push_back(str.substr(lastPos, pos - lastPos));
		// Skip delimiters.  Note the "not_of"
		lastPos = str.find_first_not_of(delimiters, pos);
		// Find next "non-delimiter"
		pos = str.find_first_of(delimiters, lastPos);
	}
}

unsigned int read_fasta(string file_name) {

	string line, last_head;
	size_t found;
	if (debug) {
		cerr << "Reading fasta '" << file_name << "'" << endl;
	}
	unsigned int hits = 0, size = 0;
	ifstream myfile(file_name.c_str());
	if (myfile.is_open()) {
		while (!myfile.eof()) {
			getline(myfile, line);

			// Crop header to what blast makes it
			if (line.substr(0, 1) == ">") {
				if (hits > 0) {
					//  protein_species_mapping
					if (proteins.count(last_head) > 0) {
						string
								spec_two =
										filenames[protein_species_mapping[proteins[last_head].first]];
						throw "Protein-ID '" + last_head
								+ "' appears twice.\nFound in '" + file_name
								+ "' as well as in '" + spec_two + "'.";
					}
					if (logfile) {
						log_out << file_name << "\t" << last_head << "\t"
								<< protein_counter << endl;
					}
					protein_species_mapping.push_back(species_counter);
					proteins[last_head] = pair<unsigned int, unsigned int> (
							protein_counter++, size);
				}
				size = 0;
				hits++;
				found = line.find(" ", 1);
				if (found > 0)
					found--;
				last_head = line.substr(1, found);
			} else {
				size += int(line.size());
			}
		}

		if (proteins.count(last_head) > 0) {
			//		string x = protein_species_mapping[proteins[last_head].first];
			throw "Protein-ID '" + last_head + "' appears twice!\nFound in '"
					+ file_name + "' as well as in ''.";
		}

		protein_species_mapping.push_back(species_counter);
		proteins[last_head] = pair<unsigned int, unsigned int> (
				protein_counter++, size);
		myfile.close();
	} else {
		throw "Unable to open fasta file '" + string(file_name) + "'";
	}

	return hits;
}

void setup_species(string list) {
	if (debug) {
		cerr << "Reading species from list in file '" << list << "'" << endl;
	}
	string line;
	ifstream listfile(list.c_str());
	if (listfile.is_open()) {
		while (!listfile.eof()) {
			getline(listfile, line);
			if (line.empty()) {
				continue;
			}
			filenames.push_back(line);
		}
		listfile.close();
	} else {
		throw "Unable to open file '" + list
				+ "' containing the list of protein databases";
	}
}

void stats(float i, float size) {
	float stat = float(int(i/size*100));
	if (stat > last_stat) {cerr << stat << "%     " << "\r"; last_stat = stat;}
}


//////////////////////////////////////////////////
// Algebraic functions
//////////////////////////////////////////////////

double getConnectivity(vector<unsigned int> nodes) {

//	cerr << "-------------------------------" << endl;
	bool check = true;
	bool did = false;

	// Knoten vom Grad 1 entfernen
	while (check) {
		check = false;
		for (unsigned int i = 0; i < nodes.size();) {
			if (graph[nodes[i]].size() == 1) {
				// Vice versa entfernen
				for (unsigned int j = 0; j < graph[graph[nodes[i]][0]].size(); j++) {
					unsigned int new_size = graph[graph[nodes[i]][0]].size()-1;
					if (graph[graph[nodes[i]][0]][j] == nodes[i]) {
						graph[graph[nodes[i]][0]][j] = graph[graph[nodes[i]][0]][new_size];
						graph[graph[nodes[i]][0]].resize(new_size);
						break;
					}
				}

				// Knoten selbst leeren
				graph[nodes[i]].clear();

				// Knoten aus eigener Liste nehmen
				nodes[i] = nodes[nodes.size()-1];
				nodes.resize(nodes.size()-1);
				// Marker resetten
				did = true;
				check = true;
				i = 0;
			}
			else {
				i++;
			}
		}
	}
	// we need to recolor the component, maybe it is a tree or ring now
	if (did) {
		return -100;
	}

	// compress value range/map data
	// list: species -> proteins
	for (unsigned int i = 0; i < nodes.size(); i++) {
			mapping[nodes[i]] = i;
	}

	// Get max degree of nodes
	unsigned int max = max_deg(nodes);

	// Get randomized x
	 vector<double> x = generate_random_vector(nodes.size());
	 double last_length;
	 double length;
	 // Orthogonalize
	 vector<double> x_hat = makeOrthogonal(x);
	 // Get lenght & normalize vector
	 vector<double> norm = nomalize(x_hat, &last_length);
//	 unsigned int iterations = 0;
	 while(1) {
	    // Gather new x
	    x = get_new_x(norm, nodes);
	    // Gather new x
	    vector<double> y = getY(max,norm,x,nodes);
	    // Orthogonalize
	    x_hat = makeOrthogonal(y);
	    // Get lenght=lambda & normalize vector
	    norm = nomalize(x_hat, &length);

	       // Check if change is to small
	       if (abs(length-last_length) < 0.001) {break;}
	       last_length = length;
//	       cerr << "Iteration: " << iterations++ << "\tScore: " << length << endl;
//	       if (++iterations > 10000) {throw "Scheint nicht zu konvergieren!";}
	   }
	   //wegen text unter formel (3)
	   //lambda_2 = 2*maxdegree - ac
	   //ac=-lambda_2+2*max_degree
	   //lambda_2 = lambda = length

	   double alg_conn = (-length+2*max)/(nodes.size());

//	   cerr << " Size: " << nodes.size() << "\tCon: " << alg_conn << endl;



	   // The set should be splitted
	   if (alg_conn < con_threshold) {
		   splitGroups(x_hat,nodes);
		   return -10-alg_conn;
	   }

	  return alg_conn;
}

void splitGroups(vector<double> y, vector<unsigned int> nodes){

	// Check to which group the nodes belong
	map<unsigned int,bool> groupA, groupB;
	for (unsigned int i = 0; i < y.size(); i++) {
//		cerr << i << " = " << y[i] << endl;
		if (y[i] < 0) 		{groupA[nodes[i]] = true;} // cerr << " " << y[i] << " => A" << endl;}
		else              	{groupB[nodes[i]] = true;} // cerr << " " << y[i] << " => B" << endl;}
		// Mark for redo
	}
//	cerr << " GroupA: " <<  groupA.size() << "\tGroupB: " <<  groupB.size() << endl;

	remove_excluded_links(groupA);	groupA.clear();
	remove_excluded_links(groupB);	groupB.clear();

}

void remove_excluded_links(map<unsigned int, bool> group) {

	    // Go through first map
//		cerr << "Remove for... " << endl;
		for(map<unsigned int, bool>::iterator it = group.begin(); it != group.end(); it++) {
//			cerr << " node: " << it->first << "(" << graph[it->first].size() << " elements)" << endl;
			// for each connected node
			for (unsigned int i = 0; i < graph[it->first].size();) {
//						  cerr << "  element: " << i << endl;
						  // check if still in the group
						  if (group.find(graph[it->first][i]) == group.end()) {
//							  cerr << " located" << endl;
							  // if not, remove the entry
							  // overwrite this with last and resize to remove last
							  graph[it->first][i] = graph[it->first][graph[it->first].size()-1];
							  graph[it->first].resize(graph[it->first].size()-1);
						  }
						  else {
							  i++;
						  }
					  }
		}

}

// Qx, Formula (5)
vector<double> getY(double max, vector<double> x_hat, vector<double> x_new, vector<unsigned int> nodes){

	// (2*maxdeg - grad_node_i ) * x_hat_i + new_x_i
	for (unsigned int i = 0; i < x_hat.size(); i++) {
		x_hat[i] *= (2*max - graph[nodes[i]].size());
		x_hat[i] += x_new[i];
	}

	return x_hat;
}

// Get biggest degree from a list of nodes sigma
unsigned int max_deg(vector<unsigned int> adj_list) {

		unsigned int max = 0;
		for (unsigned int i = 0; i < adj_list.size(); i++) {if (graph[adj_list[i]].size() > max) max = graph[adj_list[i]].size();}

		return max;
}

// determine new X, Formula (1)
vector<double> get_new_x(vector<double> x, vector<unsigned int> nodes) {
	vector<double> x_new(x.size());

	// go through all nodes (rows of A)
	for (unsigned int i = 0; i < nodes.size(); i++) {
		unsigned int node = nodes[i];	// node x is node y here
		x_new[i] = 0;
		// go through adjacency list of node (cols of A_i)
		for (unsigned int j = 0; j < graph[node].size(); j++) {
			// y points to z, so take entry z from x
			x_new[i] += x[mapping[graph[node][j]]]; // -> Sum over x entries, but transform to mapped numbers before
		}
	}

	return x_new;
}

// Generate random vector x of size size
vector<double> generate_random_vector(unsigned int size) {

	vector<double> x(size);
	for (unsigned int i = 0; i < size; i++) {
	  x[i] = (double)rand()/RAND_MAX;
		// at least one value must be different from the others
		if (i > 0 && x[i] == x[i-1]) x[i]++;
	}

	return x;
}

// Make vector x orthogonal to 1, Formula (2)
vector<double> makeOrthogonal(vector<double> x) {

	double sum = 0;
	for (unsigned int i = 0; i < x.size(); i++) {sum += x[i];}
	double average = sum/x.size();
	for (unsigned int i = 0; i < x.size(); i++) {x[i] -= average;}

	return x;
}

// Normalize vector x, Formula (4)
vector<double> nomalize(vector<double> x, double *length) {

	double sum = 0;
	for (unsigned int i = 0; i < x.size(); i++) {sum += x[i]*x[i];}
	*length = sqrt(sum);
	if (*length == 0) {*length = 0.000000001;}	// ATTENTION!
	for (unsigned int i = 0; i < x.size(); i++) {x[i] /= *length;}

	return x;
}
